package com.sample;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.impl.ClassPathResource;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.WorkItemHandler;
import org.drools.runtime.process.WorkItemManager;
import org.jbpm.process.audit.JPAProcessInstanceDbLog;
import org.jbpm.process.audit.JPAWorkingMemoryDbLogger;
import org.jbpm.process.instance.ProcessInstance;
import org.jbpm.process.instance.impl.ProcessInstanceImpl;
import org.jbpm.workflow.instance.WorkflowProcessInstance;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.process.WorkItem;

import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

public class JPATest  {
    private PoolingDataSource ds1;
    JPAProcessInstanceDbLog log;
	
    public JPATest() {
    }

    @Before
    public void setUp() throws Exception {
    	 ds1 = new PoolingDataSource();
         ds1.setUniqueName( "jdbc/testDS1" );
         ds1.setClassName( "org.h2.jdbcx.JdbcDataSource" );
         ds1.setMaxPoolSize( 3 );
         ds1.setAllowLocalTransactions( true );
         ds1.getDriverProperties().put( "user",
                                        "sa" );
         ds1.getDriverProperties().put( "password",
                                        "sasa" );
         ds1.getDriverProperties().put( "URL",
                                        "jdbc:h2:mem:mydb" );
         ds1.init();
    }

	@After
    public void tearDown() throws Exception {
        ds1.close();
    }

	private KnowledgeBase initKbase() {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(new ClassPathResource("JPATest.bpmn"), ResourceType.BPMN2);
       KnowledgeBuilderErrors errors = kbuilder.getErrors();
        if (errors.size() > 0) {
            for (KnowledgeBuilderError error : errors) {
                System.out.println(error.getMessage());

            }
            return null;
        }
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		
		return kbase;
	}

	private Environment initEnv() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory( "org.drools.persistence.jpa" );
		Environment env = KnowledgeBaseFactory.newEnvironment();
	    env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
	    env.set(EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager());
		return env;
	}


    /*
     * MyWorkItemHandler should complete the first human task to move process instance to rule task node,
     * 
     */
    @Test
    public void jpaTest() throws InterruptedException {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(new ClassPathResource("JPATest.bpmn"), ResourceType.BPMN2);
        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages( kbuilder.getKnowledgePackages() );

        Environment env = initEnv();

        StatefulKnowledgeSession ksession = JPAKnowledgeService.newStatefulKnowledgeSession( kbase, null, env );
        Assert.assertNotNull(ksession);
		KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);
        new JPAWorkingMemoryDbLogger(ksession);
	  
        MyJPAWorkItemHandler myWorkItemHandler = new MyJPAWorkItemHandler();
        System.out.println("handler" + myWorkItemHandler);
        ksession.getWorkItemManager().registerWorkItemHandler("Human Task", (WorkItemHandler) myWorkItemHandler);

        WorkflowProcessInstance process = (WorkflowProcessInstance) ksession.startProcess("JPATest");
        long processId = process.getId();
        Assert.assertNotNull(process);
        Assert.assertNotNull(process.getProcessId());
        // assert failed due to process.getKnowledgeRuntime == null
        Assert.assertNotNull(process.getKnowledgeRuntime());
        Assert.assertEquals("Task 1", process.getNodeInstances().iterator().next().getNodeName());

        System.out.println("Completing the first Activity !");
        //Complete the first human task activity
        ksession = disposeAndReloadKSession(ksession, kbase, env);
        Assert.assertNotNull(ksession);
        //ksession.getWorkItemManager().registerWorkItemHandler("Human Task", myWorkItemHandler);

		KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);
        new JPAWorkingMemoryDbLogger(ksession);
        process =  (WorkflowProcessInstance)ksession.getProcessInstance( processId);
        Assert.assertNotNull(process);
        
        // Is the Process still Active?
        Assert.assertEquals(ProcessInstance.STATE_ACTIVE, process.getState());
        // Is there a running node instance?
        Assert.assertEquals(1, process.getNodeInstances().size());
        // Is the process stopped in the "Task 1" activity?
        Assert.assertTrue(process instanceof ProcessInstanceImpl);
        Assert.assertNotNull(((ProcessInstanceImpl)process).getKnowledgeRuntime());
        Assert.assertEquals("Task 1", process.getNodeInstances().iterator().next().getNodeName());

        WorkItem workItem = myWorkItemHandler.getWorkItem();
        myWorkItemHandler = new MyJPAWorkItemHandler();
        System.out.println("handler" + myWorkItemHandler);
        ksession.getWorkItemManager().registerWorkItemHandler("Human Task", (WorkItemHandler) myWorkItemHandler);
        myWorkItemHandler.completeWorkItem(ksession.getWorkItemManager(), workItem, null);
        
        //Complete the second human activity
        System.out.println("Fetch the second Activity");
        workItem = myWorkItemHandler.getWorkItem();
        Assert.assertNotNull(workItem);
        System.out.println("Completing the second Activity");
        ksession = disposeAndReloadKSession(ksession, kbase, env);
        Assert.assertNotNull(ksession);
		KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);
        new JPAWorkingMemoryDbLogger(ksession);

        //ksession.getWorkItemManager().registerWorkItemHandler("Human Task", myWorkItemHandler);

        process =  (WorkflowProcessInstance)ksession.getProcessInstance( processId);
        Assert.assertNotNull(process);
        // Is the Process still Active?
        Assert.assertEquals(ProcessInstance.STATE_ACTIVE, process.getState());
        // Is there a running node instance?
        Assert.assertEquals(1, process.getNodeInstances().size());
        Assert.assertEquals("Task 2", process.getNodeInstances().iterator().next().getNodeName());
        
        
        myWorkItemHandler.completeWorkItem(ksession.getWorkItemManager(), workItem, null);

        try {Thread.sleep(1000);} catch (Exception e) {}
        ksession = disposeAndReloadKSession(ksession, kbase, env);
        Assert.assertNotNull(ksession);

		KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);
        new JPAWorkingMemoryDbLogger(ksession);

        process =  (WorkflowProcessInstance)ksession.getProcessInstance( processId);
        Assert.assertNull(process);
    }

    private StatefulKnowledgeSession disposeAndReloadKSession(StatefulKnowledgeSession ksession, KnowledgeBase kbase, Environment env) {
    			int ksessionId = ksession.getId();
    			ksession.dispose();
    		return JPAKnowledgeService.loadStatefulKnowledgeSession( ksessionId, kbase, null, env);
    }
    
	private StatefulKnowledgeSession createKSession(KnowledgeBase kbase, Environment env) {
	    StatefulKnowledgeSession ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null, env);
		return ksession;
	}

	private class MyJPAWorkItemHandler implements WorkItemHandler {
	    private WorkItem workItem;

	    public void completeWorkItem(WorkItemManager workItemManager, WorkItem workItem, Map<String, Object> parameters) {
	    	System.out.println("handler when complete "+ this);
	        System.out.println("complete work item " + workItem.getId());
	        workItemManager.completeWorkItem(workItem.getId(), parameters);
	    }
	    
	    public WorkItem getWorkItem() {
	    	return workItem;
	    }

		public void executeWorkItem(
				WorkItem workItem,
				WorkItemManager manager) {
	        System.out.println("handler when execute "+ this);
	        System.out.println("execute work item "+ workItem.getId());
	        this.workItem = workItem;
		}

		public void abortWorkItem(WorkItem workItem,
				WorkItemManager manager) {
	        System.out.println("abort work item");
		}
	}

}



